
from aws_ir.libs import connection
from aws_ir.libs import compromised

from aws_ir.plugins import disableaccess_key
from aws_ir.plugins import revokests_key

"""Compromise class for Key Compromise Procedure"""
class Compromise(object):

    def __init__(self,
            examiner_cidr_range='0.0.0.0/0',
            compromised_access_key_id=None,
            region='us-west-2',
            case=None,
            logger=None
        ):

        if compromised_access_key_id==None:
            raise ValueError(
                    'Must specifiy an access_key_id for the compromised key.'
            )

        self.case_type = 'Key'
        self.compromised_access_key_id = compromised_access_key_id
        self.region = region
        self.case = case
        self.logger = logger


    def mitigate(self):
        """Any steps that run as part of key compromises."""
        access_key = self.compromised_access_key_id
        compromised_resource = compromised.CompromisedMetadata(
            compromised_object_inventory = {
                'access_key_id': access_key,
                'region': self.region
            },
            case_number=self.case.case_number,
            type_of_compromise='key_compromise'
        ).data()

        client = connection.Connection(
            type='client',
            service='iam',
            region=compromised_resource['region']
        ).connect()

        self.logger.event_to_logs(
                "Attempting key disable."
        )


        # step 1 - disable access key
        disableaccess_key.Disableaccess(
            client=client,
            compromised_resource = compromised_resource,
            dry_run=False
        )


        # step 2 - revoke and STS tokens issued prior to now
        revokests_key.RevokeSTS(
            client=client,
            compromised_resource = compromised_resource,
            dry_run=False
        )

        self.logger.event_to_logs(
                "STS Tokens revoked issued prior to NOW."
        )

        self.logger.event_to_logs(
                "Disable complete.  Uploading results."
        )

        self.case.teardown(
            region=self.region,
            resource_id=self.compromised_access_key_id
        )



"""Case management operations for AWS_IR."""

import random
import logging
import sys
import os

from datetime import datetime, timedelta

from aws_ir.libs import ts_logger
from aws_ir.libs import s3bucket
from aws_ir.libs import connection
from aws_ir.libs import aws
from aws_ir.libs import inventory


class Case(object):
    """Takes case number and examiner cidr."""
    def __init__(
        self, case_number=None,
        examiner_cidr_range='0.0.0.0/0',
        case_bucket=None
    ):
        if case_number:
            self.case_number = case_number
        else:
            self.case_number = self.__generate_case_number()

        if case_bucket:
            self.case_bucket = case_bucket
        else:
            self.case_bucket = self.__setup_bucket(region='us-west-2')

        self.case_logger = Logger(self.case_number)
        self.examiner_cidr_range = examiner_cidr_range


    def prep_aws_connections(self):
        """ Get all the required information before doing the mitigation. """
        client = connection.Connection(
            type='client',
            service='ec2'
        ).connect()

        self.case_logger.event_to_logs("Initial connection to AmazonWebServices made.")

        self.amazon = aws.AmazonWebServices(
            connection.Connection(
                type='client',
                service='ec2'
            )
        )

        self.available_regions = self.amazon.regions

        self.case_logger.event_to_logs("Inventory AWS Regions Complete {region_count} found.".format(
                region_count = len(self.amazon.regions)
            )
        )

        self.availability_zones = self.amazon.availability_zones

        self.case_logger.event_to_logs(
                "Inventory Availability Zones Complete {zone_count} found.".format(
                zone_count = len(self.amazon.availability_zones)
            )
        )

        self.case_logger.event_to_logs(
                "Beginning inventory of resources world wide.  This might take a minute..."
        )

        self.aws_inventory = inventory.Inventory(
            connection.Connection(
                type='client',
                service='ec2'
            ),
            self.available_regions
        )

        self.case_logger.event_to_logs(
                "Inventory complete.  Proceeding to resource identification."
        )

        self.inventory = self.aws_inventory.inventory


    def __rename_log_file(self, case_number, resource_id):
        """Move all log files to standard naming format"""
        try:
            os.rename(
                ("/tmp/{case_number}-aws_ir.log").format(
                    case_number=case_number,
                    ),
                ("/tmp/{case_number}-{resource_id}-aws_ir.log").format(
                    case_number=case_number,
                    resource_id=resource_id
                    )
            )
            return True
        except:
            return False

    def copy_logs_to_s3(self, bucket):
        """Convinience function to put all case logs to s3 at the end"""
        s3 = boto3.resource('s3')
        case_bucket = s3.Bucket(self.bucket)
        logs = self.get_case_logs()
        for log in logs:
            case_bucket.upload_file(str("/tmp/" + log), log)

    def teardown(self, region, resource_id):
        """ Any final post mitigation steps universal to all plans. """
        try:
            self.__rename_log_file(self.case_number, resource_id)
            #self.copy_logs_to_s3()
            processing_end_messaging = (
                """Processing complete for {case_number}"""
            ).format(case_number=self.case_number)
            print(processing_end_messaging)
            sys.exit(0)
        except Exception as e:
            sys.exit(0)
            raise


    def get_case_logs(self):
        """Enumerates all case logs based on case number from system /tmp"""
        files = []
        for file in os.listdir("/tmp"):
            if file.startswith(self.case_number):
                files.append(file)
        return files


    def __setup_bucket(self, region):
        """Wrap s3 find or create in object"""
        bucket_name = s3bucket.CaseBucket(
                self.case_number,
                region
        ).bucket.name

        return bucket_name


    def __generate_case_number(self):
        return datetime.utcnow().strftime(
            'cr-%y-%m%d%H-{0:04x}'
            ).format(
                random.randint(0,2**16)
        )

class Logger(object):
    """Case logger class for wrapping output formatters."""
    def __init__(self, case_number=None, add_handler=False, verbose=False):
        """Setup the stream logger for the object"""
        self.case_number = case_number
        self.logger = logging.getLogger('aws_ir.cli')
        self.verbose = verbose
        if self.verbose:
            self.logger.setLevel(logging.DEBUG)
        else:
            self.logger.setLevel(logging.INFO)
        streamhandler = logging.StreamHandler(sys.stdout)

        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        )

        if add_handler == True:
            streamhandler.setFormatter(formatter)
            self.logger.addHandler(streamhandler)
        else:
            pass
            #There is already a stream handler


    def event_to_logs(self, message):
        """Use timesketch logger format to create custody chain"""
        try:
            json_event = ts_logger.timesketch_logger(message, self.case_number)
            self.logger.info(message)
            return True
        except:
            return False



import boto3

class AmazonWebServices(object):
    def __init__(self, client):
        self.client = client
        self.regions = self.__get_regions()
        self.availability_zones = self.__get_availability_zones()

    def __get_regions(self):
        """ Use the provided AWS Client to iterate over the regions and store them """
        availRegions = []
        regions = self.client.connect().describe_regions()
        for region in regions['Regions']:
            availRegions.append(region['RegionName'])
        return availRegions

    def __get_availability_zones(self):
        """ Use the provided AWS Client to iterate over the azs and store them """
        availZones = []
        for region in self.regions:
            self.client.region = region
            client = self.client.connect()
            zones = client.describe_availability_zones()['AvailabilityZones']
            for zone in zones:
                if zone['State'] == 'available':
                    availZones.append(zone['ZoneName'])
        return availZones



import uuid
import boto3

class Isolate(object):
    def __init__(
        self,
        client,
        compromised_resource,
        dry_run
    ):

        self.client = client
        self.compromised_resource = compromised_resource
        self.compromise_type = compromised_resource['compromise_type']
        self.examiner_cidr_range = compromised_resource['examiner_cidr_range']
        self.dry_run = dry_run

        self.setup()

    def setup(self):
        sg = self.__create_isolation_security_group()
        acl = self.__create_network_acl()
        connections_stopped = self.__add_network_acl_entries(acl)

        """Conditions that can not be dry_run"""
        if self.dry_run == False:
            self.__add_security_group_rule(sg)
            egress_revoked = self.__revoke_egress(sg)
            instance_shifted = self.__add_security_group_to_instance(sg)
        pass

    def validate(self):
        """Validate that the instance is in fact isolated"""
        if self.sg_name != None:
            return True
        else:
            return False

    def __create_isolation_security_group(self):
        try:
            security_group_result = self.client.create_security_group(
                DryRun=self.dry_run,
                GroupName=self.__generate_security_group_name(),
                Description="ThreatResponse Isolation Security Group",
                VpcId=self.compromised_resource['vpc_id'],

            )

        except Exception as e:
            try:
                if e.response['Error']['Message'] == """
                Request would have succeeded, but DryRun flag is set.
                """:
                    security_group_result['GroupId'] = None
            except:
                raise e
        return security_group_result['GroupId']

    def __add_security_group_rule(self, security_group_id):
        try:
            self.client.authorize_security_group_ingress(
                DryRun=self.dry_run,
                GroupId=security_group_id,
                IpProtocol='tcp',
                FromPort=22,
                ToPort=22,
                CidrIp=self.examiner_cidr_range
            )
        except Exception as e:
            try:
                if e.response['Error']['Message'] == """
                Request would have succeeded, but DryRun flag is set.
                """:
                    pass
            except:
                raise e

    def __revoke_egress(self, group_id):
        try:
            response = self.client.revoke_security_group_egress(
                DryRun=self.dry_run,
                GroupId=group_id
            )

            return True
        except:
            return False

    def __generate_security_group_name(self):
        sg_name = "isolation-sg-{case_number}-{instance}-{uuid}".format(
            case_number=self.compromised_resource['case_number'],
            instance=self.compromised_resource['instance_id'],
            uuid=str(uuid.uuid4())
        )
        self.sg_name = sg_name
        return sg_name

    def __add_security_group_to_instance(self, group_id):
        try:
            response = self.client.modify_instance_attribute(
                DryRun=self.dry_run,
                InstanceId=self.compromised_resource['instance_id'],
                Groups=[
                    group_id,
                ],
            )
            return True
        except:
            return False

    def __create_network_acl(self):
        try:
            response = self.client.create_network_acl(
                DryRun=self.dry_run,
                VpcId=self.compromised_resource['vpc_id'],
            )
            return response['NetworkAcl']['NetworkAclId']
        except Exception as e:
            try:
                if e.response['Error']['Message'] == """
                Request would have succeeded, but DryRun flag is set.
                """:
                    return None
            except:
                raise e

    def __add_network_acl_entries(self, acl_id):
        try:
            response = self.client.create_network_acl_entry(
                DryRun=self.dry_run,
                NetworkAclId=acl_id,
                RuleNumber=1300,
                Protocol='-1',
                RuleAction='allow',
                Egress=False,
                CidrBlock=self.examiner_cidr_range
            )

            response = self.client.create_network_acl_entry(
                DryRun=self.dry_run,
                NetworkAclId=acl_id,
                RuleNumber=1337,
                Protocol='-1',
                RuleAction='deny',
                Egress=True,
                CidrBlock="{compromised_host_private_ip}/32".format(
                    compromised_host_private_ip=self.compromised_resource['private_ip_address']
                )
            )
            return True
        except Exception as e:
            try:
                if e.response['Error']['Message'] == """
                Request would have succeeded, but DryRun flag is set.
                """:
                    return None
            except:
                raise e



import boto3

class Snapshotdisks(object):
    def __init__(
        self,
        client,
        compromised_resource,
        dry_run
    ):

        self.client = client
        self.compromised_resource = compromised_resource
        self.compromise_type = compromised_resource['compromise_type']
        self.dry_run = dry_run

        self.session = boto3.Session(
                    region_name=self.compromised_resource['region']
        )

        self.setup()

    def setup(self):
        self.__snapshot_volumes()

    def __create_snapshot(self, volume_id, description):
        try:
            response = self.client.create_snapshot(
                DryRun=self.dry_run,
                VolumeId=volume_id,
                Description=description
            )
            return response
        except Exception as e:
            if e.response['Error']['Message'] == """
            Request would have succeeded, but DryRun flag is set.
            """:
                return None
            else:
                return e

    def __tag_snapshot(self, snapshot_id):
        ec2 = self.session.resource('ec2')
        snapshot = ec2.Snapshot(snapshot_id)
        snapshot.create_tags(
            Tags=[
                dict(
                    Key='cr-case-number',
                    Value=self.compromised_resource['case_number']
                )
            ]
        )

    def __snapshot_volumes(self):
        for volume_id in self.compromised_resource['volume_ids']:
            description = 'Snapshot of {vid} for case {cn}'.format(
                vid=volume_id,
                cn=self.compromised_resource['case_number']
            )

            snapshot = self.__create_snapshot(volume_id, description)
            if snapshot != None:
                snapshot_id = snapshot['SnapshotId']
                self.__tag_snapshot(snapshot_id)
        pass



import boto3
class Connection(object):
    def __init__(self, type, service, region=None):
        self.region = region
        self.connection_type = type
        self.service = service
        self.client = None
        self.resource = None

    def connect(self):
        if self.connection_type is None:
            raise StandardError(
                "Could not determine connect type.  Set client or resource."
            )
        elif self.connection_type == "client":
            client = boto3.client(
                self.service,
                region_name=self.region
            )
            self.client = client
            return self.client
        elif self.connection_type == "resource":
            resource = boto3.resource(
                self.service,
                region_name=self.region
            )
            self.resource = resource
            return self.resource
        else:
            raise StandardError(
                "Connection type is not supported."
            )



#!/usr/bin/env python
import sys
import argparse

from aws_ir import __version__
from aws_ir.libs import case

#Support for multiple incident plans coming soon
from aws_ir.plans import key
from aws_ir.plans import host

"""Basic arg parser for AWS_IR cli"""
class cli():
    def __init__(self):
        self.config = None
        self.prog = sys.argv[0].split('/')[-1]

    """Throw an error on missing modules"""
    def module_missing(self, module_name):
        try:
            __import__(module_name)
        except ImportError as e:
            return True
        else:
            return False

    """Parent parser for top level flags"""
    def parse_args(self, args):

        parser = argparse.ArgumentParser(
            description="""
                Incident Response command line for Amazon Web Services.
                This command line interface is designed to process host and key
                based incursions without delay or error.
            """
        )

        optional_args = parser.add_argument_group()

        optional_args.add_argument(
            '--version',
            action='version',
            version="%(prog)s {ver}".format(ver=__version__))

        optional_args.add_argument(
            '--verbose',
            action='store_true',
            help='log debug messages')

        optional_args.add_argument(
            '--case-number',
            default=None,
            help="""
                The case number to use., usually of the form "cr-16-053018-2d2d"
            """
        )

        optional_args.add_argument(
            '--examiner-cidr-range',
            default='0.0.0.0/0',
            help="""
                The IP/CIDR for the examiner and/or the tool.
                This will be added as the only allowed range
                in the isolated security group.
            """
        )

        optional_args.add_argument(
            '--bucket-name',
            default=None,
            help="""
                Optional.
                The id of the s3 bucket to use.
                This must already exist
            """
        )

        optional_args.add_argument(
            '--dry-run',
            action='store_true',
            help="""
                Dry run. Pass dry run
                parameter to perform API calls
                but will not modify any resources.
            """
        )

        subparsers = parser.add_subparsers(dest="compromise-type")
        subparsers.required = True

        host_compromise_parser = subparsers.add_parser(
            'host-compromise', help=''
        )

        host_compromise_parser.add_argument(
            '--instance-ip',
            required=True,
            help='')
        host_compromise_parser.add_argument(
            '--user',
            required=True,
            help="""
                this is the privileged ssh user
                for acquiring memory from the instance.
            """
        )
        host_compromise_parser.add_argument(
            '--ssh-key',
            required=True,
            help='provide the path to the ssh private key for the user.'
        )
        host_compromise_parser.set_defaults(func="host_compromise")

        key_compromise_parser = subparsers.add_parser(
            'key-compromise',
            help=''
        )

        key_compromise_parser.add_argument(
            '--access-key-id', required=True, help=''
        )

        key_compromise_parser.set_defaults(func="key_compromise")

        return parser.parse_args(args)


    """Logic to decide on host or key compromise"""
    def run(self):
        self.config = self.parse_args(sys.argv[1:])
        case_logger = case.Logger(add_handler=True, verbose=self.config.verbose)
        case_logger.event_to_logs("Parsing successful proceeding to incident plan.")
        compromise_object = None
        if self.config.func == 'host_compromise':
            hc = host.Compromise(
                user = self.config.user,
                ssh_key_file = self.config.ssh_key,
                compromised_host_ip = self.config.instance_ip,
                prog = self.prog,
                case = case.Case(
                    self.config.case_number,
                    self.config.examiner_cidr_range,
                    self.config.bucket_name

                ),
                logger = case_logger
            )
            compromise_object = hc
            try:
                hc.mitigate()
            except KeyboardInterrupt:
                pass
        elif self.config.func == 'key_compromise':
            kc = key.Compromise(
                self.config.examiner_cidr_range,
                self.config.access_key_id,
                case = case.Case(
                    self.config.case_number,
                    self.config.examiner_cidr_range,
                    self.config.bucket_name

                ),
                logger = case_logger
            )

            compromise_object = kc
            try:
                kc.mitigate()
            except KeyboardInterrupt:
                pass


if __name__=='__main__':
   c = cli()
   if c.prog is not None:
       c.run()



import base64
import boto3

class DisableOwnKeyError(RuntimeError):
    """ Thrown when a request is made to disable the current key being used.  """
    pass

class Disableaccess(object):
    def __init__(
        self,
        client,
        compromised_resource,
        dry_run
    ):

        self.client = client #Requires an IAM Client
        self.compromised_resource = compromised_resource
        self.compromise_type = compromised_resource['compromise_type']
        self.dry_run = dry_run

        self.access_key_id = self.compromised_resource['access_key_id']
        self.setup()

    def setup(self):
        """Method runs the plugin"""
        if self.dry_run != True:
            self.__disable_access_key()

    def validate(self):
        """Returns whether this plugin does what it claims to have done"""
        client = self.client
        response = client.get_access_key_last_used(AccessKeyId=self.access_key_id)
        username = response['UserName']
        access_keys = client.list_access_keys(
            UserName=username
        )

        for key in access_keys['AccessKeyMetadata']:
            if (key['AccessKeyId'] == self.access_key_id) and (key['Status'] == 'Inactive'):
                return True

        return False

    def __disable_access_key(self, force_disable_self=False):
        """This function first checks to see if the key is already disabled\
        if not then it goes to disabling"""
        client = self.client
        #First check to see if the key is already disabled.
        if self.validate == True:
            return
        else:
            # we get the username for the key because even though username is optional
            # if the username is not provided, the key will not be found, contrary to what
            # the documentation says.
            response = client.get_access_key_last_used(AccessKeyId=self.access_key_id)
            username = response['UserName']
            client.update_access_key(
                UserName=username,
                AccessKeyId=self.access_key_id,
                Status='Inactive'
            )



import uuid
import boto3

class Tag(object):
    def __init__(
        self,
        client,
        compromised_resource,
        dry_run
    ):
        self.client = client
        self.compromised_resource = compromised_resource
        self.compromise_type = compromised_resource['compromise_type']
        self.dry_run = dry_run
        self.tagged = self.__add_incident_tag_to_instance()

    def validate(self):
        instance_id = self.compromised_resource['instance_id']
        response = self.client.describe_instances(
            InstanceIds=[instance_id],
            Filters=[
                {
                    'Name': 'tag-key',
                    'Values': [
                        'cr-case-number'
                    ]
                },
            ],
        )
        if len(response['Reservations']) > 0:
            return True
        else:
            return False

    def __create_tags(self):
        tag = [
            {
                'Key': 'cr-case-number',
                'Value': self.compromised_resource['case_number']
            }
        ]

        return tag

    def __add_incident_tag_to_instance(self):
        instance_id = self.compromised_resource['instance_id']
        try:
            self.client.create_tags(
                DryRun=self.dry_run,
                Resources=[instance_id],
                Tags=self.__create_tags()
            )
            return True

        except Exception as e:
            if e.response['Error']['Message'] == """
                Request would have succeeded, but DryRun flag is set.
                """:
                return None
            else:
                raise e





import margaritashotgun

class Memory(object):
    def __init__(
        self,
        compromised_resource,
        dry_run,
        client=None,
        verbose=False
    ):

        self.client = client
        self.compromised_resource = compromised_resource
        self.compromise_type = compromised_resource['compromise_type']
        self.dry_run = dry_run
        self.verbose = verbose

    def get_memory(
        self,
        bucket,
        ip,
        user,
        key,
        case_number,
        port=None,
        password=None
    ):
        name = 'margaritashotgun'
        config = dict(aws = dict(bucket = bucket),
                      hosts = [ dict(addr = ip, port = port,
                                    username = user,
                                    password = password,
                                    key = key) ],
                      workers = 'auto',
                      logging = { 'dir': '/tmp/',
                                  'prefix':  ("{case_number}-{ip}").format(
                                                ip=ip,
                                                case_number=case_number ) },
                      repository = dict(enabled = True,
                                        url = ('https://threatresponse-lime'
                                               '-modules.s3.amazonaws.com/')))
        capture_client = margaritashotgun.client(name=name, config=config,
                                                 library=True, verbose=self.verbose)
        return capture_client.run()



import base64
import boto3

"""Gathers ephemeral data that could be lost on instance termination"""

class Gather(object):
    """
        Initializer takes standard plugin constructor
        added api flag for data persistence in AWS_IR
        api
    """
    def __init__(
        self,
        client,
        compromised_resource,
        dry_run,
        api=False
    ):

        self.client = client
        self.compromised_resource = compromised_resource
        self.compromise_type = compromised_resource['compromise_type']
        self.dry_run = dry_run
        self.api = api

        """
            These attrs will only be set during API=True
            Added for Readability and AWS_IR data persistence
        """
        self.evidence = {}

        self.setup()

    def setup(self):
        if self.dry_run != True:
            metadata = self.__get_aws_instance_metadata()
            self.__log_aws_instance_metadata(metadata)
            console = self.__get_aws_instance_console_output()
            self.__log_aws_instance_console_output(console)
            self.__log_aws_instance_screenshot()
            return True
        else:
            return False

    def validate(self):
        """ Can't really validate data gather """
        return True

    def __get_aws_instance_metadata(self):
        metadata = self.client.describe_instances(
                         Filters=[
                            {
                                'Name': 'instance-id',
                                'Values': [
                                    self.compromised_resource['instance_id']
                                ]
                            }
                        ]
                    )['Reservations']

        return metadata

    def __log_aws_instance_metadata(self, data):
        if self.api == True:
            self.evidence['metadata.json'] = data
        else:
            logfile = ("/tmp/{case_number}-{instance_id}-metadata.log").format(
                case_number=self.compromised_resource['case_number'],
                instance_id=self.compromised_resource['instance_id']
            )
            with open(logfile,'w') as w:
                w.write(str(data))

    def __get_aws_instance_console_output(self):
        output = self.client.get_console_output(
            InstanceId=self.compromised_resource['instance_id']
        )
        return output

    def __log_aws_instance_console_output(self, data):
        if self.api == True:
            self.evidence['console.json'] = data
        else:
            logfile = ("/tmp/{case_number}-{instance_id}-console.log").format(
                case_number=self.compromised_resource['case_number'],
                instance_id=self.compromised_resource['instance_id']
            )
            with open(logfile,'w') as w:
                w.write(str(data))

    def __log_aws_instance_screenshot(self):
        response = self.client.get_console_screenshot(
               InstanceId=self.compromised_resource['instance_id'],
               WakeUp=True
        )
        if self.api == True:
            self.evidence['screenshot.jpg'] = response['ImageData']
        else:
            logfile = ("/tmp/{case_number}-{instance_id}-screenshot.jpg").format(
                case_number=self.compromised_resource['case_number'],
                instance_id=self.compromised_resource['instance_id']
            )

            fh = open(logfile, "wb")
            fh.write(base64.b64decode(response['ImageData']))
            fh.close()



from distutils.command.build import build
from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def run(self):
        self.run_command('build')
        _install.run(self)

setup(name="aws_ir",
      version="0.2.1",
      author="Andrew Krug, Alex McCormack, Joel Ferrier",
      author_email="andrewkrug@gmail.com,developer@amccormack.net,joel@ferrier.io",
      packages=["aws_ir", "aws_ir/libs", "aws_ir/plans", "aws_ir/plugins"],
      license="MIT",
      description="AWS Incident Response ToolKit",
      scripts=['bin/aws_ir'],
      url='https://github.com/ThreatResponse/aws_ir',
      download_url="https://github.com/ThreatResponse/aws_ir/archive/v0.2.1.tar.gz",
      use_2to3=True,
      install_requires=['boto3>=1.3.0',
                        'progressbar_latest',
                        'logutils',
                        'requests',
                        'structlog',
                        'pytz',
                        'jinja2',
                        'margaritashotgun>=0.4.0'],
      tests_require=['pytest',
                     'pytest-cov',
                     'moto',
                     'mock',
                     'magicmock'],
      )



import os
import sys
import boto3
import json
import pprint
import time

if sys.version_info[0] == 3:
    import urllib.request as request
else:
    import urllib2 as request
from datetime import datetime, timedelta

class Query(object):
  def __init__(self, client, regions, inventory_type):
    self.client = client
    self.regions = regions

    if inventory_type == "edda":
        self.result = self.get_all_running_from_edda()
    else:
        self.result = self.get_all_running_from_aws()

  def get_running_by_region_aws(self, region):
    inventory = []
    self.client.region = region
    client = self.client.connect()

    reservations = client.describe_instances(
          Filters=[{'Name': 'instance-state-name', 'Values': ['running']}]
          )['Reservations']

    for reservation in reservations:
        for instance in reservation['Instances']:
            instance_data = self.__extract_data_aws(instance, region)
            instance_data['region'] = region
            inventory.append(instance_data)
    return inventory

  def get_running_by_region_edda(self, region):
    inventory = []

    edda_url = self.__edda_url()
    api_call = "{edda_url}view/instances/;_all".format(edda_url=edda_url)
    reservations = request.urlopen(api_call)
    reservations = json.loads(reservations.read().decode())
    for instance in reservations:
        instance_data = self.__extract_data_edda(instance)
        if instance_data['region'] == region:
            inventory.append(instance_data)

    return inventory

  def __edda_url(self):
      server = os.environ.get('EDDA_SERVER')
      port = os.environ.get('EDDA_PORT')
      base_url = os.environ.get('EDDA_BASE_URL')

      edda_url = "http://{server}:{port}{base_url}".format(
        server=server,
        port=port,
        base_url=base_url
      )
      return edda_url

  def __extract_data_aws(self, instance, region):
    return dict(
        public_ip_address = instance.get('PublicIpAddress', None),
        private_ip_address = instance.get('PrivateIpAddress', None),
        instance_id = instance['InstanceId'],
        launch_time = instance['LaunchTime'],
        platform = instance.get('Platform', None),
        vpc_id = instance['VpcId'],
        ami_id = instance['ImageId'],
        volume_ids = [ bdm['Ebs']['VolumeId'] for bdm in instance.get('BlockDeviceMappings', [] ) ],
        region = region
    )

  def __extract_data_edda(self, instance):
    return dict(
        public_ip_address = instance.get('publicIpAddress', None),
        private_ip_address = instance.get('privateIpAddress', None),
        instance_id = instance['instanceId'],
        launch_time =  instance['launchTime'],
        platform = instance.get('platform', None),
        vpc_id = instance['vpcId'],
        ami_id = instance['imageId'],
        volume_ids = [ bdm['ebs']['volumeId'] for bdm in instance.get('blockDeviceMappings', [] ) ],
        region = str(instance['placement']['availabilityZone'])[:-1]
    )

  def get_all_running_from_aws(self):
    inventory = {}
    for region in self.regions:
        inventory[region] = self.get_running_by_region_aws(region)
    return inventory

  def get_all_running_from_edda(self):
    inventory = {}
    for region in self.regions:
        inventory[region] = self.get_running_by_region_edda(region)
    return inventory

class InventoryType(object):
    def __init__(self):
        self.type = self.__inventory_type()

    def __inventory_type(self):
        if os.environ.get('EDDA_SERVER'):
            return "edda"
        else:
            return "aws"

class Inventory(object):
  def __init__(self, client, regions):
    self.client = client
    self.regions = regions

    self.type = InventoryType().type

    self.inventory = Query(
        self.client,
        self.regions,
        self.type
    ).result

  def locate_instance(self, ip):
      for region in self.regions:
        located = list(
            filter(
                lambda x: x['public_ip_address'] == ip,
                self.inventory[region]
            )
        )

        if len(located) == 0:
            pass
        else:
          return located[0]



class CompromisedMetadata(object):
    def __init__(
        self,
        compromised_object_inventory,
        case_number,
        type_of_compromise,
        examiner_cidr_range='0.0.0.0/0'
    ):

        self.inventory = compromised_object_inventory
        self.case_number = case_number
        self.examiner_cidr_range = examiner_cidr_range
        self.compromise_type = type_of_compromise

    def data(self):
        metadata = self.inventory
        metadata['case_number'] = self.case_number
        metadata['compromise_type'] = self.compromise_type
        metadata['examiner_cidr_range'] = self.examiner_cidr_range
        return metadata



import boto3

class Stop(object):
    def __init__(
        self,
        client,
        compromised_resource,
        dry_run
    ):

        self.client = client
        self.compromised_resource = compromised_resource
        self.compromise_type = compromised_resource['compromise_type']
        self.dry_run = dry_run
        self.setup()

    def setup(self):
        if self.dry_run != True:
            self.__stop_instance()
        else:
            pass

    def __stop_instance(self):
        response = self.client.stop_instances(
            InstanceIds=[
                self.compromised_resource['instance_id']
            ],
            Force=True
        )
        return response



import datetime
import boto3
import os
import fnmatch

from jinja2 import Template

class RevokeSTS(object):
    def __init__(
        self,
        client,
        compromised_resource,
        dry_run
    ):

            self.client = client
            self.compromised_resource = compromised_resource
            self.compromise_type = compromised_resource['compromise_type']
            self.dry_run = dry_run

            self.setup()

    def setup(self):
        """Method runs the plugin attaching policies to the user in question"""
        if self.dry_run == False:
            username = self.__get_username_for_key()
            policy_document = self.__generate_inline_policy()
            self.__attach_inline_policy(username, policy_document)
            pass
        pass

    def validate(self):
        """Checks the a policy is actually attached"""
        for policy in self.__get_policies()['PolicyNames']:
            if policy == "threatresponse-temporal-key-revocation":
                return True
            else:
                pass
        return False

    def __get_policies(self):
        """Returns all the policy names for a given user"""
        username = self.__get_username_for_key()
        policies = self.client.list_user_policies(
            UserName=username
        )
        return policies

    def __get_date(self):
        """Returns a date in zulu time"""
        now = datetime.datetime.utcnow().isoformat() + 'Z'
        return now

    def __get_username_for_key(self):
        """Find the user for a given access key"""
        client = self.client
        response = client.get_access_key_last_used(
            AccessKeyId=self.compromised_resource['access_key_id']
        )
        username = response['UserName']
        return username

    def __generate_inline_policy(self):
        """Renders a policy from a jinja template"""
        template_name = self.__locate_file('deny-sts-before-time.json.j2')
        template_file = open(template_name)
        template_contents = template_file.read()
        jinja_template = Template(template_contents)
        policy_document = jinja_template.render(
            before_date=self.__get_date()
        )
        return policy_document

    def __attach_inline_policy(self, username, policy_document):
        """Attaches the policy to the user"""
        client = self.client

        response = client.put_user_policy(
            UserName=username,
            PolicyName="threatresponse-temporal-key-revocation",
            PolicyDocument=policy_document
        )
        return response

    def __locate_file(self, pattern, root=os.curdir):
        '''Locate all files matching supplied filename pattern in and below
        supplied root directory.'''
        for path, dirs, files in os.walk(os.path.abspath(root)):
            for filename in fnmatch.filter(files, pattern):
                return os.path.join(path, filename)



import time
import pytz
import sys
import logging
import structlog
import json
import os
from structlog import get_logger, configure
from structlog import wrap_logger
from structlog import ReturnLogger
from structlog import PrintLogger
from structlog.stdlib import filter_by_level
from structlog.processors import JSONRenderer
from datetime import datetime

class timesketch_logger(object):
    def __init__(self, message, case_number):
        #Logging setup for json only
        self.message = message
        self.timestamp = int(time.time())
        self.datetime = datetime.utcnow().isoformat()
        self.desc = "AWS_IR Action"


        configure(
            processors=[JSONRenderer(indent=1, sort_keys=True)],
            context_class=structlog.threadlocal.wrap_dict(dict),
            logger_factory=structlog.stdlib.LoggerFactory()
        )

        self.log = get_logger('aws_ir.json')

        event = ReturnLogger().msg(
            'message',
            message=self.message,
            timestamp=self.timestamp,
            datetime=self.datetime,
            desc=self.desc
        )


        def generate_log_filename(case_number):
            filename = ("/tmp/{case_number}-aws_ir.log").format(case_number=case_number)
            return filename

        def log_file_exists():
            exists = os.path.isfile(self.logfile)
            return exists

        self.logfile = generate_log_filename(case_number)

        def file_len(fname):
           i = 0
           with open(fname) as f:
               for i, l in enumerate(f):
                   pass
           return i + 1

        def log_file_contains_events(logfile):
            length = file_len(logfile)
            if length >= 2:
                return True
            else:
                return False

    def stub_ts_file(logfile):
            with open(logfile, "w") as f:
                f.write("[ \n")
            f.close()

    def write_log_event(event):
        logfile = self.logfile
        if log_file_exists():
            if (log_file_contains_events(logfile) == True):
                f = open(logfile)
                lines = f.readlines()
                f.close()
                with open(logfile,'w') as w:
                    w.writelines([item for item in lines[:-1]])
                    w.write("\t" + str(event) + ",")
                    w.write("\n")
                    w.write("]")
                    w.close()
            else:
               stub_ts_file(logfile)
               with open(logfile,'a') as w:
                   w.write("\t" + str(event) + ",")
                   w.write("\n")
                   w.write("]")
                   w.close()
        else:
            stub_ts_file(logfile)
            with open(logfile,'a') as w:
               w.write("\t" + str(event) + ",")
               w.write("\n")
               w.write("]")
               w.close()
        write_log_event(event[1])



import os

from aws_ir.libs import volatile
from aws_ir.libs import compromised
from aws_ir.libs import connection

from aws_ir.plugins import isolate_host
from aws_ir.plugins import tag_host
from aws_ir.plugins import gather_host
from aws_ir.plugins import snapshotdisks_host
from aws_ir.plugins import stop_host


"""Compromise class for Host Compromise"""
class Compromise(object):
    """ Procedures for responding to a HostCompromise.
        ** For now, assume only Linux system.
    """
    def __init__(
            self,
            user=None,
            ssh_key_file=None,
            compromised_host_ip=None,
            prog=None,
            case=None,
            logger=None
        ):


        if compromised_host_ip==None:
            raise ValueError(
                'Must specifiy an IP for the compromised host'
            )

        if not os.path.exists(ssh_key_file):
            raise ValueError(
                'Key file must exist. Could not find path {0}'.format(
                        ssh_key_file
                )
            )

        self.prog = prog
        self.ssh_key_file_path = ssh_key_file
        self.user = user
        self.case_type = 'Host'

        self.compromised_host_ip = compromised_host_ip
        self.case = case
        self.logger = logger

    def mitigate(self):

        self.case.prep_aws_connections()

        search = self.case.aws_inventory.locate_instance(self.compromised_host_ip)

        if search == None:
            raise ValueError('Compromised IP Address not found in inventory.')

        compromised_resource = compromised.CompromisedMetadata(
            compromised_object_inventory = search,
            case_number=self.case.case_number,
            type_of_compromise='host_compromise',
            examiner_cidr_range=self.case.examiner_cidr_range
        ).data()

        client = connection.Connection(
            type='client',
            service='ec2',
            region=compromised_resource['region']
        ).connect()


        # step 1 - isolate
        isolate_host.Isolate(
            client=client,
            compromised_resource = compromised_resource,
            dry_run=False
        )

        # step 2 - apply compromised tag
        tag_host.Tag(
            client=client,
            compromised_resource = compromised_resource,
            dry_run=False
        )

        #step 3 - get instance metadata and store it
        gather_host.Gather(
            client=client,
            compromised_resource = compromised_resource,
            dry_run=False
        )


        # step 4 - create snapshot
        snapshotdisks_host.Snapshotdisks(
            client=client,
            compromised_resource = compromised_resource,
            dry_run=False
        )

        # step 5 - gather memory
        if compromised_resource['platform'] == 'windows':
            self.logger.event_to_logs('Platform is Windows skipping live memory')
        else:
            self.logger.event_to_logs(
                "Attempting run margarita shotgun for {user} on {ip} with {keyfile}".format(
                    user=self.user,
                    ip=self.compromised_host_ip,
                    keyfile=self.ssh_key_file_path
                    )
                )
            try:
                volatile_data = volatile.Memory(
                    client=client,
                    compromised_resource = compromised_resource,
                    dry_run=False,
                    verbose=self.logger.verbose
                )

                results = volatile_data.get_memory(
                      bucket=self.case.case_bucket,
                      ip=self.compromised_host_ip,
                      user=self.user,
                      key=self.ssh_key_file_path,
                      case_number=self.case.case_number
                 )

                self.logger.event_to_logs(("memory capture completed for: {0}, "
                                    "failed for: {1}".format(results['completed'],
                                                             results['failed'])))
            except Exception as ex:
                # raise keyboard interrupt passed during memory capture
                if isinstance(ex, KeyboardInterrupt):
                    raise
                else:
                    self.logger.event_to_logs(
                        (
                            "Memory acquisition failure with exception"
                              "{exception}. ".format(
                                                exception=ex
                            )
                        )
                    )

        # step 6 - shutdown instance
        stop_host.Stop(
            client=client,
            compromised_resource = compromised_resource,
            dry_run=False
        )

        self.case.teardown(
            region=compromised_resource['region'],
            resource_id=compromised_resource['instance_id']
        )



import boto3
import uuid

"""Class to create the cases s3 bucket for asset storage"""
class CaseBucket(object):
    def __init__(self, case_number, region):
        self.region = region
        self.case_number = case_number
        self.client = boto3.client(
            's3',
            region_name=region
            )
        self.session = boto3.Session(
             region_name=region
        )
        self.s3 = self.session.resource('s3')
        self.bucket = self.find_or_create_by()

    def find_or_create_by(self):
        bucket = self.__locate_bucket()
        if bucket is not None:
            return bucket
        else:
            self.bucket_name = self.__generate_name()
            bucket = self.__create_s3_bucket()
            self.__set_acls(self.bucket_name)
            self.__set_tags(self.bucket_name)
            self.__set_versioning(self.bucket_name)
            return bucket
        pass

    def cleanup_empty_buckets(self):
        buckets = self.client.list_buckets()
        for bucket in buckets['Buckets']:
            if str(bucket['Name']).find('cloud-response') != -1:
                try:
                    self.client.delete_bucket(Bucket=bucket['Name'])
                    print(bucket['Name'])
                except:
                    pass


    def __generate_name(self):
        bucket_name = 'cloud-response-' + str(uuid.uuid4()).replace('-','')
        return bucket_name

    def __create_s3_bucket(self):
        # the if statement is to prevent a fun little bug https://github.com/boto/boto3/issues/125
        if self.region == 'us-east-1':
          bucket = self.s3.create_bucket(
            Bucket=self.bucket_name
          )
        else:
            bucket = self.s3.create_bucket(
                Bucket=self.bucket_name,
                CreateBucketConfiguration={
                    'LocationConstraint': self.region
                }
            )
        return bucket

    def __set_acls(self, bucket_name):
        self.s3.BucketAcl(bucket_name).put(ACL='bucket-owner-full-control')

    def __set_tags(self, bucket_name):
        self.client.put_bucket_tagging(
            Bucket=bucket_name,
            Tagging=dict(
                TagSet=[
                    dict(
                        Key='cr-case-number',
                        Value=self.case_number
                    )
                ]
            )
        )

    def __set_versioning(self, bucket_name):
        self.client.put_bucket_versioning(
            Bucket=bucket_name,
            VersioningConfiguration=dict(
                MFADelete='Disabled',
                Status='Enabled'
                )
            )

    def __locate_bucket(self):
        buckets = self.s3.buckets.all()
        for bucket in buckets:
          if bucket.name.startswith("cloud-response-"):
              tags = self.__get_bucket_tags(bucket.name)
              if self.__check_tags(tags):
                  case_bucket = bucket
                  return case_bucket
              else:
                  return None
          else:
              pass

    def __get_bucket_tags(self, bucket):
        try:
            s3 = boto3.client('s3')
            response = s3.get_bucket_tagging(
                Bucket=bucket,
            )
        except:
            response = None
        return response

    def __check_tags(self, tag_object):
        if tag_object is None:
            return False
        for tag in tag_object['TagSet']:
            if tag['Value'] == self.case_number:
               return True
            else:
               return False
